/*############################################################################
#                                                                            #
#             I2CLN^[LCDuAQM0802A-RN-GBWvphCo                #
#                                                                            #
#@RpCmFFAtmelStudio 6.1(SP2)                                  #
#                                                                            #
#                                                  2014/02/17 Create by nPYO #
#@mFW[F                                                      #
#@@Hʏ uAQM0802A-RN-GBWv(8x2s)                                   #
#                                                                            #
############################################################################*/
#define __LCD_ST7032_C__

#include <util/delay.h>
#include "lcd_st7032.h"
#include "twi_functions.h"

/* Zbg|[g`			*/
#if USE_RESET
#define RESET_DDR  ( *(volatile uint8_t *)( &RESET_PIN + 0x01 ) )
#define RESET_PORT ( *(volatile uint8_t *)( &RESET_PIN + 0x02 ) )
#endif

/* obNCg|[g`		*/
#if USE_BACKLIGHT
#define BACKLIGHT_DDR  ( *(volatile uint8_t *)( &BACKLIGHT_PIN + 0x01 ) )
#define BACKLIGHT_PORT ( *(volatile uint8_t *)( &BACKLIGHT_PIN + 0x02 ) )
#endif

#define LCD_CMD 0x00
#define LCD_DATA 0x40

/* obNCg					*/
static uint8_t backLight_status = OFF;

/*****************************************************************************
*								LCD f[^o
* Fuint8_t rs   0x00(LCD_DATA):f[^ 0x40(LCD_CMD):R}h
*		uint8_t data o̓f[^
* ߒlFvoid
*****************************************************************************/
static void lcd_Write( uint8_t rs ,uint8_t data )
{
	if ( TWI_transferStart( ST7032_TWI_ADDR ,TWI_WRITE ) == 0 ) {
		if ( TWI_write( rs ) == TWI_ACK ) {
			TWI_write( data );
		}
		TWI_transferEnd();
		_delay_us( 50 );
	}
}

/*****************************************************************************
*								LCD
* Fuint8_t contrast RgXgl( 3.3V 30 x )
* ߒlFvoid
*****************************************************************************/
void lcdInit( uint8_t contrast )
{
	/* LCD Zbg			*/
	#if USE_RESET
		RESET_PORT &= ~_BV(RESET_BIT);
		_delay_ms( 1 );
		RESET_PORT |= _BV(RESET_BIT);
	#endif

	/* LCD R}h	*/
	_delay_ms( 50 );
	lcd_Write( LCD_CMD , 0x38 );					/* function set			*/
	lcd_Write( LCD_CMD , 0x39 );					/* function set			*/
	lcd_Write( LCD_CMD , 0x14 );					/* interval osc			*/
	lcd_Write( LCD_CMD , 0x70 | ( contrast & 0xf ) );			/* contrast Low						*/
	lcd_Write( LCD_CMD , 0x5c | ( ( contrast >> 4 ) & 0x3 ) );	/* contast High/icon/power			*/
	lcd_Write( LCD_CMD , 0x6c );					/* follower control		*/
	_delay_ms( 300 );
	lcd_Write( LCD_CMD , 0x38 );					/* function set			*/
	lcd_Write( LCD_CMD , 0x0c );					/* display on			*/
	lcd_Write( LCD_CMD , 0x01 );					/* clear display		*/
	_delay_ms( 2 );

	lcdLocate( 0 ,0 );
}

/*****************************************************************************
*								obNCgԎ擾
* Fvoid
* ߒlFuiknt8_t ON/OFF
*****************************************************************************/
uint8_t lcdBackLightStatus( void )
{
	return ( backLight_status );
}

/*****************************************************************************
*								obNCg
* Fuint8_t sw ON / OFF
* ߒlFvoid
*****************************************************************************/
void lcdBackLight( uint8_t sw )
{
	if ( sw == OFF ) {
	#if USE_BACKLIGHT
		BACKLIGHT_PORT &= ~_BV( BACKLIGHT_BIT );
	#endif
		backLight_status = OFF;
	} else {
	#if USE_BACKLIGHT
		BACKLIGHT_PORT |= _BV( BACKLIGHT_BIT );
	#endif
		backLight_status = ON;
	}
}

/*****************************************************************************
*								Oo^
* Fuint8_t o^ԍ(0`7)
*		uint8_t data[] rbgf[^(8bytes)
* ߒlFvoid
*****************************************************************************/
void lcdSetChar( uint8_t idx ,uint8_t *data )
{
	lcd_Write( LCD_CMD ,0x40 | ( ( idx & 0x03 ) << 3 ) );
	for ( uint8_t i = 8 ;i ;i-- ) {
		lcd_Write( LCD_DATA ,*data++ );
	}
}

/*****************************************************************************
*								ʃNA
* Fvoid
* ߒlFvoid
*****************************************************************************/
void lcdCls( void )
{
	lcd_Write( LCD_CMD ,0x01 );					/* clear display			*/
    _delay_ms( 2 );
	lcdLocate( 0 ,0 );
}

/*****************************************************************************
*							J[\^CvύX
* FLCD_CURSOL_TYPE type J[\^Cv
*							 LCD_CURSOL_OFF      : J[\\
*							 LCD_CURCOL_BLINK    : ubN_
*							 LCD_CURSOL_UNDERBAR : A_[o[
* ߒlFvoid
*****************************************************************************/
void lcdCursol( LCD_CURSOL_TYPE type )
{
	lcd_Write( LCD_CMD ,0x0c | ( type & 0x03 ) );
}

/*****************************************************************************
*								J[\ړ
* Fuint8_t row		sԍ(0`)
*		uint8_t column	ԍ(0`)
* ߒlFvoid
*****************************************************************************/
void lcdLocate( uint8_t row ,uint8_t column )
{
	if ( row == 1 ) column += 0x40;
	lcd_Write( LCD_CMD ,0x80 | column );
}

/*****************************************************************************
*								Po
* Fchar c         R[h
* ߒlFvoid
*****************************************************************************/
void lcdPutc( char c )
{
	lcd_Write( LCD_DATA ,(uint8_t)c );
}

/*****************************************************************************
*						RAM̈̕o
*	Fchar *s  
*	ߒlF
*****************************************************************************/
void lcdPuts( char *s )
{
	while ( *s ) {
		lcdPutc( *s++ );
	}
}

/*****************************************************************************
*						R[ḧ̕o
*	Fconst char *s 
*	ߒlFvoid
*
*	ex. lcdPuts_P( PSTR( "Test!!" ) );
*****************************************************************************/
void lcdPuts_P( const char *s ) 
{
	char c;
	while ( ( c = pgm_read_byte( s++ ) ) ) {
		lcdPutc( c );
	}
}
