/*############################################################################
#  INA226-Test( USB_meter )                                                  #
#                                                                            #
#@RpCmFFAtmelStudio 6.1(SP2)                                  #
#                                                                            #
#                                                  2014/02/17 Create by nPYO #
############################################################################*/
#define __MAIN_C__

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <stdio.h>

#include "lcd_st7032.h"
#include "ina226.h"
#include "twi_functions.h"

/* {^Ԏ擾}N ( 0:ON !0:OFF )	*/
#define PUSH_SW ( PIND & _BV(PD6) )

/* [J֐vg^Cv錾				*/
static void hardwareInit( void );

/*****************************************************************************
*  荞݃xN^
*						^C}[1I[o[t[
*****************************************************************************/
ISR( TIMER1_OVF_vect)
{
	/* Ȃ(X[vpj	*/
}

/*****************************************************************************
*								n[hEGA
* Fvoid
* ߒlFvoid
*****************************************************************************/
static void hardwareInit( void )
{
	wdt_disable();		/* ZbgÕEIb`hbN^C}[ݒ𖳌ɂ	*/

	/*  PortB +--------7: Pullup : N.C.                                         */
	/*        |+-------6: Pullup : N.C.                                         */
	/*        ||+------5: Out(H) : SPI-SCK                                      */
	/*        |||+-----4: HI-Z   : SPI-MISO                                     */
	/*        ||||+----3: Out(H) : SPI-MOSI                                     */
	/*        |||||+---2: Pullup : N.C.                                         */
	/*        ||||||+--1: Pullup : N.C.                                         */
	/*        |||||||+-0: Pullup : N.C.                                         */
	DDRB  = 0b00101000;
	PORTB = 0b11101111;
	
	/*  PortC +--------7: ------ :                                              */
	/*        |+-------6: ------ :                                              */
	/*        ||+------5: HI-Z   : I2C-SCL                                      */
	/*        |||+-----4: HI-Z   : I2C-SDA                                      */
	/*        ||||+----3: Pullup : PC3                                          */
	/*        |||||+---2: Pullup : PC2                                          */
	/*        ||||||+--1: Pullup : PC1                                          */
	/*        |||||||+-0: Pullup : PC0                                          */
	DDRC  = 0b00000000;
	PORTC = 0b11001111;

	/*  PortD +--------7: Pullup : N.C.                                         */
	/*        |+-------6: Pullup : Push Switch                                  */
	/*        ||+------5: Pullup : INA226 #Alert                                */
	/*        |||+-----4: Pullup : N.C.                                         */
	/*        ||||+----3: Out(L) : LCD #Reset                                   */
	/*        |||||+---2: Pullup : N.C.                                         */
	/*        ||||||+--1: Pullup : N.C.                                         */
	/*        |||||||+-0: Pullup : N.C.                                         */
	DDRD  = 0b00001000;
	PORTD = 0b11110111;

	/* AiORp[^			*/
	ACSR = _BV(ACD);

	/* ȓd͐ݒ(gȂ@\OFFj	*/
	PRR = _BV(PRTIM2) | _BV(PRTIM0) | _BV(PRSPI) | _BV(PRUSART0) | _BV(PRADC);

	/* ^C}[0(8bit)ݒ				*/
	TCCR1B = _BV(CS12) | _BV(CS10);				/* 8MHz/1024=7812.5Hz	*/
	TIMSK1 |= _BV(TOV1);						/* I[o[t[ݗL	*/

	/* EIb`hbN^C}[ݒ		*/
	wdt_enable( WDTO_1S );

	/* TWI 						*/
	TWI_init();

	/* LCD						*/
	lcdInit( 30 );

	/* INA226 					*/
	ina226_write( INA226_REG_CONFIGURATION ,0x8000 );			/* Zbg									*/

	uint16_t con = ina226_read();

	con &= 0xf1ff;												/* ϒl߂鎞̑񐔐ݒ			*/
	con |= INA226_AVG4 << 9;									/* 񐔂قǈ肷邪x͒x	*/

	ina226_write( INA226_REG_CONFIGURATION , con );				/* ConfigurationWX^ݒ				*/

	ina226_write( INA226_REG_CALIBRATION ,INA226_CALIBRATION );	/* Lu[VWX^ݒ			*/

	/* X[v						*/
	set_sleep_mode( SLEEP_MODE_IDLE );
	sleep_enable();

	/* O[oC^vgL		*/
	sei();
}

/*****************************************************************************
*									C֐
* Fvoid
* ߒlFint ---
*****************************************************************************/
int main( void )
{
	static const char tbl[] PROGMEM = { 0x00 ,'|' ,'/' ,'-' };	/* ]Aj[Ve[u	*/
	char buf[ 10 ];
	short offset = 0;

	/* eytF						*/
	hardwareInit();

	/* ]Aj[Vp̃LN^iobNXbVj`	*/
	buf[ 0 ] = 0x00;
	buf[ 1 ] = 0x10;
	buf[ 2 ] = 0x08;
	buf[ 3 ] = 0x04;
	buf[ 4 ] = 0x02;
	buf[ 5 ] = 0x01;
	buf[ 6 ] = 0x00;
	buf[ 7 ] = 0x00;
	lcdSetChar( 0 ,(uint8_t *)buf );

	/* C[v									*/
	uint8_t stp = 0;								/* Aj[VJE^		*/
	while ( 1 ) {

		/* EEPɕۑꂽItZbgl擾				*/
		eeprom_busy_wait();
		eeprom_read_block( &offset ,(void *)0x0000 ,sizeof(short) );

		/* ItZbgl\(1b)					*/
		lcdCls();
		lcdPuts_P( PSTR("Offset") );
		sprintf_P( buf ,PSTR("%d") ,offset );
		lcdLocate( 1 ,0 );
		lcdPuts( buf );

		for ( uint16_t t = 1000U ;t ;t-- ) {
			wdt_reset();
			_delay_ms( 1 );
		}

		/* {^Ă΃[v				*/
		while ( !PUSH_SW ) wdt_reset();

		/* E\C[v					*/
		while ( 1 ) {
			/* EIb`hbNZbg				*/
			wdt_reset();

			/* Aj[V\				*/
			lcdLocate( 0 ,0 );
			lcdPutc( pgm_read_byte( &tbl[ stp ] ) );
			if ( ++stp >= sizeof(tbl) ) stp = 0;

#if 0
			/* INA226 d͒l\					*/
			ina226_set_pointer( INA226_REG_POWER );
			uint32_t power = (uint32_t)ina226_read() * 25UL;		/* 1Count = 25mW	*/
			sprintf_P( buf ,PSTR("%5dmW") ,power );
			lcdPuts( buf );
#endif

#if 1
			/* INA226 dl\					*/
			ina226_set_pointer( INA226_REG_BUS_VOLTAGE );
			uint32_t voltage = (uint32_t)ina226_read() * 125UL;		/* 1Count = 1.25mV			*/
			uint16_t n = (uint16_t)( ( voltage + 500UL ) / 1000UL );
			sprintf_P( buf ,PSTR("%3u.%02uV") ,n / 100U ,n % 100U );
			lcdPuts( buf );
#endif

			/* INA226 dl\					*/
#if 0
			ina226_set_pointer( INA226_REG_CURRENT );
			short current = (short)ina226_read();				/* 1Count = 1mA					*/
			if ( current < 0 ) current = 0;						/* ̏ꍇ0mA\			*/
#endif

#if 1
			ina226_set_pointer( INA226_REG_SHUNT_VOLTAGE );
			short sv = (short)ina226_read() - offset;			/* 1Count = 2.5V				*/
			if ( sv < 0 ) sv = 0;
			uint16_t current = (uint16_t)( ( (uint32_t)sv * 2500UL ) / (uint32_t)INA226_SHUNT_R / 1000UL );
#endif

			sprintf_P( buf ,PSTR("%6dmA") ,current );
			lcdLocate( 1 ,0 );
			lcdPuts( buf );										/* dl					*/

			/* ItZbglL						*/
			if ( !PUSH_SW ) {									/* {^ꂽH			*/
				ina226_set_pointer( INA226_REG_SHUNT_VOLTAGE );
				short offset = (short)ina226_read();
				eeprom_busy_wait();
				eeprom_write_block( &offset ,(void *)0x0000 ,sizeof(short) );
				break;
			}

			/* CPU X[v(300msec)				*/
			TCNT1 = 65535U - (uint16_t)( F_CPU * 300 / 1024000 );
			sleep_cpu();
		}

	}
	return ( 0 );
}
